/*
# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Santé)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------   
*/

#ifdef _USE_WXWIDGETS_

//==========================================================================
// WITH WX
//==========================================================================

#include "bbtkwxGUIEditorGraphicBBS.h"

#include <wx/cmdline.h>
#include <wx/app.h>
#include <wx/sysopt.h>

#include "vtkOutputWindow.h"
#include "vtkRenderWindow.h"

class vtkOutputWindowbbGEditor : public vtkOutputWindow
{
public:
	vtkOutputWindowbbGEditor() {}
	static vtkOutputWindowbbGEditor* New() { return new vtkOutputWindowbbGEditor;}
	virtual void 	DisplayDebugText (const char *) {}
	virtual void 	DisplayText (const char *) {}
	virtual void 	DisplayErrorText (const char *) {}
	virtual void 	DisplayWarningText (const char *) {}
	virtual void 	DisplayGenericWarningText (const char *) {}
protected:
};





//EED 2017-09-16 Migration wxWidgets 2.8 to 3.0
#if wxMAJOR_VERSION <= 2
	static const wxCmdLineEntryDesc cmdLineDesc[] =
	{
	{ wxCMD_LINE_PARAM,  NULL, NULL, _T("file"), wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL }, 
	{ wxCMD_LINE_SWITCH, _T("h"), _T("help"),   _T("Prints this help") },
	//{ wxCMD_LINE_SWITCH, _T("d"), _T("debug"),   _T("Message all 9") },
	{ wxCMD_LINE_NONE }
	};
#else
	static const wxCmdLineEntryDesc cmdLineDesc[] =
	{
	{ wxCMD_LINE_PARAM,  NULL, NULL, "file", wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL }, 
	{ wxCMD_LINE_SWITCH, "h", "help",   "Prints this help" },
	//{ wxCMD_LINE_SWITCH, _T("d"), _T("debug"),   _T("Message all 9") },
	{ wxCMD_LINE_NONE }
	};
#endif




class wxBBEditorApp : public wxApp
{
public:
  bool OnInit( );
  int  OnExit() { return true; }

	 bool usage;
	bool OnCmdLineParsed(wxCmdLineParser& parser);
	void OnInitCmdLine(wxCmdLineParser& parser);
private:
	 std::vector<std::string> input_file;

};

IMPLEMENT_APP(wxBBEditorApp);



void wxBBEditorApp::OnInitCmdLine(wxCmdLineParser& parser)
{
	//    std::cout << "wxBBEditorApp::OnInitCmdLine"<<std::endl;
	parser.SetDesc(cmdLineDesc);
}


bool wxBBEditorApp::OnCmdLineParsed(wxCmdLineParser& parser)
{
	int argc = parser.GetParamCount();
	for (int i=0; i<argc; ++i) 
    {
		std::string s = bbtk::wx2std(parser.GetParam(i));
		input_file.push_back(s);
    }
	
	bool help = ( parser.Found(_T("h")) );
	usage = (help && (input_file.size()==0));
	if (usage) {
		std::cout << "bbEditor (The Black Box Development Graphic Editor) - bbtk "
		<< bbtk::GetVersion() << " - (c) Creatis 2013"
		<< std::endl;
		parser.Usage();
	}
	return true;
}


// ----------------------------------------------------------------------------
// The `main program' equivalent, creating the windows and returning the
// main frame

#define GL_MAJOR_VERSION 0x821B
#define GL_MINOR_VERSION 0x821C

bool wxBBEditorApp::OnInit( )
{	
	vtkRenderWindow *renWin = vtkRenderWindow::New();
    
    printf("EED wxBBEditorApp::OnInit >>>>> vtkRenderWindow IsDirect       %d\n", renWin->IsDirect() );

/* EED  2022-07-26 Segmentation fault with VTK9.2 in MacOs
	int supportopengl=renWin->SupportsOpenGL();
	printf("EED wxBBEditorApp::OnInit >>>>> vtkRenderWindow SupportsOpenGL %d\n", supportopengl );
	renWin->Delete();
	if (supportopengl==0)
	{
#if defined(_WIN32)
		wxString message=wxT("OpenGL 3>= not detected.\n Try to install opengl drivers or use de opengl-mesa version.\n Copy from: <ProgramsFiles>\\CreaTools\\crea_TPdlls-4.0.0\\bin\\opengl-mesa the file: opengl32.dll\n to\n <ProgramsFiles>\\CreaTools\\CreaTools-3.0.0\\bin ");
#else
		wxString message=wxT("OpenGL 3>= not detected.\n Try to install opengl drivers or use de opengl-mesa version.\n ");	
#endif
		wxMessageBox(message);
		exit(0);
	}
*/
	
	vtkOutputWindowbbGEditor *vtkoutputwindowbbgeditor= vtkOutputWindowbbGEditor::New();
	vtkOutputWindow::SetInstance( vtkoutputwindowbbgeditor );
	vtkoutputwindowbbgeditor->Delete();

#ifdef MACOSX
	/* assume this is OSX */
	wxSystemOptions::SetOption("mac.listctrl.always_use_generic", 1);
#endif

	wxApp::OnInit();
#ifdef __WXGTK__
  //See http://www.wxwindows.org/faqgtk.htm#locale
  setlocale(LC_NUMERIC, "C");
#endif

	bbtk::wxGUIEditorGraphicBBS *iegbbs;

	iegbbs = new bbtk::wxGUIEditorGraphicBBS(NULL);
	SetTopWindow(iegbbs);
	iegbbs->Show(true);


	// EED  ..To Do..
	// 1. This block have to be mooved to the library
	// 2. We have to verifiy if the file is a good bbg  or a good bbs
	// 3. Clean the path to pout just the name of the file in the second parameter (information of the tabPanel)
	//	OpenDiagram(<PATH+Filename>, <Filena>); 
        for(int i=0;i<input_file.size();i++)
	{
		int strsize=input_file[i].size()-1;
		if ((input_file[i])[strsize]=='g')
		{
			iegbbs->OpenDiagram(input_file[i], input_file[i]); 
		} 
		if ((input_file[i])[strsize]=='s')
		{
			iegbbs->OpenBBS(input_file[i], input_file[i]); 
		} 
	}
	return true;
}


#if defined(_WIN32)

//  How to have a Console and wxWidgets
//  http://www.wxwidgets.org/wiki/index.php/MSVC_Setup_Guide
//   In Visual C++ 6 (7 should be similar), to create an application that is both a console application
//  (cout's to the console are visible) and has a wxWidgets GUI,
//  you need to use the linker option "/subsystem:console" and the following code:
int main(int argc, char* argv[])
{
	char buffer[500];
	wcstombs(buffer, ::GetCommandLine(), 500);
	return WinMain(::GetModuleHandle(NULL), NULL, buffer, SW_SHOWNORMAL);
//	return WinMain(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), SW_SHOWNORMAL);
}

#endif // defined(_WIN32)


#else
//==========================================================================
// WITHOUT WX
//==========================================================================
#include <iostream>
int main(int argc, char* argv[])
{
  std::cout << "bbStudio was not compiled with wxWidgets : ciao !" <<std::endl;
  return 0;
}

// EOF
#endif //#ifdef _USE_WXWIDGETS_



