/*
# ---------------------------------------------------------------------
#
# Copyright (c) CREATIS (Centre de Recherche en Acquisition et Traitement de l'Image 
#                        pour la Santé)
# Authors : Eduardo Davila, Frederic Cervenansky, Claire Mouton
#
#  This software is governed by the CeCILL-B license under French law and 
#  abiding by the rules of distribution of free software. You can  use, 
#  modify and/ or redistribute the software under the terms of the CeCILL-B 
#  license as circulated by CEA, CNRS and INRIA at the following URL 
#  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
#  or in the file LICENSE.txt.
#
#  As a counterpart to the access to the source code and  rights to copy,
#  modify and redistribute granted by the license, users are provided only
#  with a limited warranty  and the software's author,  the holder of the
#  economic rights,  and the successive licensors  have only  limited
#  liability. 
#
#  The fact that you are presently reading this means that you have had
#  knowledge of the CeCILL-B license and that you accept its terms.
# ------------------------------------------------------------------------  
*/

/*=========================================================================                                                                               
Program:   bbtk
Module:    $RCSfile$
Language:  C++
Date:      $Date$
Version:   $Revision$
=========================================================================*/

/* ---------------------------------------------------------------------

* Copyright (c) CREATIS-LRMN (Centre de Recherche en Imagerie Medicale)
* Authors : Eduardo Davila, Laurent Guigues, Jean-Pierre Roux
*
*  This software is governed by the CeCILL-B license under French law and 
*  abiding by the rules of distribution of free software. You can  use, 
*  modify and/ or redistribute the software under the terms of the CeCILL-B 
*  license as circulated by CEA, CNRS and INRIA at the following URL 
*  http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.html 
*  or in the file LICENSE.txt.
*
*  As a counterpart to the access to the source code and  rights to copy,
*  modify and redistribute granted by the license, users are provided only
*  with a limited warranty  and the software's author,  the holder of the
*  economic rights,  and the successive licensors  have only  limited
*  liability. 
*
*  The fact that you are presently reading this means that you have had
*  knowledge of the CeCILL-B license and that you accept its terms.
* ------------------------------------------------------------------------ */                                                                         

/****
* Design and Developpement of BBTK GEditor
* Ricardo A Corredor J <ra.corredor67@uniandes.edu.co>
* RaC - 2010
****/

#ifndef __GBoxModel_h__
#define __GBoxModel_h__

//Includes same project
#include "GlobalConstants.h"
#include "GObjectModel.h"
#include "GPortModel.h"

//Includes creaMaracasVisu

//Includes std
#include <iostream>


namespace bbtk
{

	// Mandatory when exists a bidirectional relation: GPortModel includes GBoxModel and viceversa
	class GPortModel;

	class GBoxModel : public GObjectModel 
	{

	public: 

		//Constructors
		GBoxModel();
		~GBoxModel();
		
		//Public methods		
		virtual void setInicPoint(double& x, double& y, double& z);

		void addInputPort(GPortModel *inputport);
		void addOutputPort(GPortModel *outputport);

		int getNumInputPorts();
		int getNumOutputPorts();

		// Change the position of the box and its ports
		virtual void move(double xx,double yy,double zz);

		// Text in the status bar
		virtual std::string getStatusText();

		// Method to write the information of the black box to be saved in the parameter string
		virtual void save(std::string &content);

		// Returns a vector with inputs or outputs
		std::vector<GPortModel*> getInputPorts();
		std::vector<GPortModel*> getOutputPorts();

		// Get input with the name given by parameter
		GPortModel* getInputPort(std::string name);
		GPortModel* getInputPort(int pos);

		// Get output with the name given by parameter
		GPortModel* getOutputPort(std::string name);
		GPortModel* getOutputPort(int pos);
		
		// Updates the position and finally the view of inputs and outputs
		void updatePorts();

		void addColons(std::string &text);
		void removeColons(std::string &text);
	private:
		//Private Attributes
		//Private Methods
	protected:
		//Protected Attributes
		std::vector<GPortModel*> _inputs;
		std::vector<GPortModel*> _outputs;
		//Protected methods
	};


}
// namespace bbtk
#endif

